﻿//Created by Björn Dagerman 2015/05 (dagerman@kth.se)
//
//Updated for 2018 course version 2017/12 (jean.malm@mdh.se)
//Main
namespace Demo
module Main =
    //This main function loops using async and Async.Await. See lecture F13 for alternatives.
    let rec loop event list = async{
        //At the start we do the computations that we can do with the inputs we have, just as in a regular application
        printfn "%A" list
    
        //Next, since we don't have all inputs (yet) we need to wait for them to become available (Async.Await)
        //let! (bang) enables execution to continue on other computations and threads.
        let! somethingObservable = Async.AwaitEvent(event) //Replace AwaitEvent with AwaitObservable if you're using a newer target platform

        //Now that we have recieved a new input we can perform the rest of our computations
        match somethingObservable with
        | 0     -> return! loop event ("0"::list)
        | 1     -> return! loop event ("1"::list)
        | _     -> return! loop event ("?"::list) //etc...

        //The last thing we do is a recursive call to ourselves, thus looping
    }

    //The GUIInterface is tightly coupled with the main loop which is its only intented user, thus the nested module
    module GUIInterface = 
        //Here we define what we will be observing (clicks)
        let events = 
            Event.merge 
            <| Event.map (fun _-> 0) GUI.btnAdd0.Click //The map transforms the observation (click) by the given function. In our case this means
            <| Event.map (fun _-> 1) GUI.btnAdd1.Click //that clicking the button AddX will return X. Note the type of observables : IObservable<int>

    //Starts the main loop and opens the GUI
    //Revision note: To run it from F# interactive, you may have to use GUI.form.ShowDialog() instead of System.Windows.Forms.Application.Run
    Async.StartImmediate(loop GUIInterface.events []) ; System.Windows.Forms.Application.Run(GUI.form)
    //Async.StartImmediate(loop GUIInterface.events []) ; GUI.form.ShowDialog()
